<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

; Include scales and the two UDOs
#include "scales.inc"

; Retrieve and print the version of the scale quantiser
instr Version
	iMajor, iMinor m_scale_get_version
	prints("\nScale quantiser version %d.%d", iMajor, iMinor)
endin

instr Midi_demo
	; Setup, for the opcode search for MIDIDEMO
	; Choose two scales from scales.inc
	kScales[] = fillarray(2, 12)
	Sscale init "" ; name of the scale

	; Set a slowly changing index, to change scales
	kScaleIndex = int(wrap(line(0, p3, 2), 0, 1.99))
	kCurScale = kScales[kScaleIndex]
	kRootNote init 0 ; Root note of C

	; An LFO to go through MIDI notes
	kLFO = oscil(12, 2/p3, -1)
	kNotes = int(kLFO + 60)

	; MIDIDEMO quantise notes to the scales
	; Print scale name
	if (changed(kCurScale) == 1) then
		Sscale = m_scale_get_name(kCurScale)
		printks("\nQuantising to %s", 1/kr, Sscale)
	endif

	; Quantise
	kPlayedNote = m_scale_midi(kNotes, kCurScale, kRootNote)

	; Run an oscillator, converting MIDI notes to cps
	aOut = vco2(.5, cpsmidinn(kPlayedNote), 0)

	; Output the oscillator
	outs(aOut, aOut)
endin

; Frequency scaler demo
instr Cps_demo
	; Setup, for the opcode part search for CPSDEMO
	; Choose two scales and an index to go through them
	kScales[] = fillarray(1, 2)
	kScaleIndex = int(wrap(line(0, p3, 2), 0, 1.99))
	kCurScale = kScales[kScaleIndex]
	kRoot init 9 ; root note at A
	Sscale init "" ; name of the current scale

	; Generate random frequencies
	kInCps = 330 + randh(110, 5) ; a new frequency each .2seconds

	; CPSDEMO Quantise the random pitches to scale
	; Print scale name
	if (changed(kCurScale) == 1) then
		Sscale = m_scale_get_name(kCurScale)
		printks("\nQuantising to %s", 1/kr, Sscale)
	endif
	kPlayedCps = m_scale_cps(kInCps, kCurScale, kRoot)

	; Play pitches through an oscillator
	aOut = vco2(.5, kPlayedCps, 0)

	; Output the oscillator
	outs(aOut, aOut)
endin

</CsInstruments>
; ==============================================
<CsScore>
i"Version" 0 3
i"Midi_demo" 3 8
i"Cps_demo" 11 8
e
</CsScore>
</CsoundSynthesizer>
