/*
m_scale_cps - quantise arbirary pitches to notes in a western scale

DESCRIPTION
Quantise arbitrary pitches in Hz (cycles per second) to a western scale with
a given root note (grade).
Scales are described in simple, but long tables. Scales are index.
The name to a corresponding scale index can be retrieve with m_scale_get_name .

SYNTAX
kNote m_scale_cps kInputFreq, kScale, kRootGrade

INITIALIZATION

PERFORMANCE
kNote - The quantised frequency
kInputFreq - Arbitrary frequency in Hz
kScale - Index of the scale to quantise to
kRootGrade - Grade or note number of an octave, 0=C, 1=C#,... 11=B

CREDITS
Author: Jeanette C.
*/

opcode m_scale_cps, k, KKK
	kInputFreq, kScale, kRootGrade xin

	; Initialise parameters
	kInputFreq init -1
	kScale init -1
	kRootGrade init -1

	; Set up internal variables
	iNumberOfScales = ftlen(giScales) / 12
	kOutputNote init 0 ; Frequency to be output
	kCurOffset init 0 ; offset into the scales ftable
	kRealRoot init 0
	kPch init 0 ; input converted to pch
	kOct init 0 ; octave of the input note
	kFrac init 0 ; fractional part of pch class representation
	kIndex init 0 ; Index into the scale array
	kLocalScale[] init 12 ; holding scale transposed by root grade

	; Only calculate a new scale, when scale or root grade have changed
	if (changed(kScale, kRootGrade) != 0) then
		; Check that a valid scale has been specified
		if (kScale < iNumberOfScales) then
			kCurOffset = kScale * 12 ; offset into the scales ftable
			kRealRoot = limit(int(kRootGrade), 0, 11)

			; Transpose scale
			kIndex = 0
			while (kIndex < 12) do
				kLocalScale[kIndex] = ((tab((kIndex + kCurOffset), giScales, 0) + kRootGrade) % 12)
				kIndex += 1
			od
			kLocalScale = sorta(kLocalScale)
		endif // correct scale ftable
	endif // Scale calculation

	; Convert input frequency to oct format and extract octave and fractional point
	kPch = octcps(kInputFreq)
	kOct = int(kPch)
	kFrac = int(frac(kPch) * 12)

	; Construct the quantised frequency, by calculating a new pch value and
	; converting that to cps
	kOutputNote = cpspch(kOct + (kLocalScale[kFrac] / 100))

	; Output quantised frequency
	xout(kOutputNote)
endop
